@@ -6,10 +6,11 @@ package Prima;
 use strict;
 use warnings;
 require DynaLoader;
-use vars qw($VERSION @ISA $__import @preload);
+use vars qw($VERSION @ISA $__import @preload $pid);
 @ISA = qw(DynaLoader);
 sub dl_load_flags { 0x00 }
-$VERSION = '1.51';
+$VERSION = '1.52';
+$pid = $$;
 bootstrap Prima $VERSION;
 unless ( UNIVERSAL::can('Prima', 'init')) {
 	$::application = 0;
@@ -57,9 +58,11 @@ sub parse_argv
 
 Prima::init($VERSION) unless $^C;
 
+sub CLONE { $pid = 0 }
+
 sub END
 {
-	&Prima::cleanup() if UNIVERSAL::can('Prima', 'cleanup');
+	&Prima::cleanup() if $pid == $$ && UNIVERSAL::can('Prima', 'cleanup');
 }
 
 sub run
@@ -397,10 +400,12 @@ L<Prima::StdDlg> - wrapper module to the toolkit standard dialogs
 
 L<Prima::Drawable::CurvedText> - fit text to path
 
-L<Prima::Drawable::Subcanvas> - paint a hierarchy of widgets to any drawable
+L<Prima::Drawable::Gradient> - gradient fills for primitives
 
 L<Prima::Drawable::Path> - stroke and fill complex paths
 
+L<Prima::Drawable::Subcanvas> - paint a hierarchy of widgets to any drawable
+
 =item Visual Builder
 
 L<VB> - Visual Builder for the Prima toolkit
