@@ -77,6 +77,7 @@ sub init {
 		(map { $_ => $profile{$_}} keys %hdrProps),
 		%{$profile{headerProfile}},
 	);
+	$self-> {header}-> send_to_back;
 	my $x = $self->{header}->items;
 	$self->{umap} = [ 0 .. $#$x];
 	$self->$_( $profile{$_}) for qw(autoRecalc columns mainColumn);
@@ -308,6 +309,7 @@ sub item_sort {
 	for my $i (@$items) {
 		if (defined $i->[1]) { $self->item_sort($i->[1], $col, $dir) }
 	}
+	$self->reset_item_cache;
 }
 
 sub on_expand {
