@@ -984,10 +984,12 @@ sub class
 	my $index = $1 || 0;
 	my $i = Prima::Icon-> create;
 	undef($i) unless $i-> load(Prima::Utils::find_image($image), index => $index);
+	my $s = $::application ? $::application->uiScaling : 1;
+	$i->ui_scale if $i;
 	return $action, {
 		class   => 'Prima::SpeedButton',
 		profile => {
-			size        => [ 24, 24],
+			size        => [ 24 * $s, 24 * $s],
 			image       => $i,
 			borderWidth => 1,
 			onClick     => \&on_click,
