@@ -530,7 +530,11 @@ sub parse_struc
 			( $type eq 'char*')  { $def = 'C_STRING_UNDEF'; } elsif
 			( $type eq 'string') { $def = 'C_STRING_UNDEF'; };
 			$tok = gettok;
-			if ( $tok eq '=') {
+			if ( $tok eq 'with' ) {
+				expect('undef');
+				expect(';');
+				$def = "undef:$def";
+			} elsif ( $tok eq '=') {
 				$def = gettok;
 				expect(';');
 			} else {
@@ -661,10 +665,12 @@ sub preload_method
 	#checking 'proc(void) & proc()' cases
 	$tok = getidsym(")*...");
 	$acc .= ', ' unless ($tok eq "void") || ($tok eq ")") || (! $useHandle);
+	my $isVoid;
 	if ($tok eq "void") {
 		if ($words[-1] eq "*") {
 			$acc .= ', ';
 		} else {
+			$isVoid = 1;
 			expect(')');
 			$tok = ')';
 		}
@@ -751,6 +757,7 @@ sub preload_method
 		error "Ellipsis (...) could be defined only in the end of parameters list"
 			if $hasEllipsis > 1 || ( $hasEllipsis == 1 && $ellipsis == 0);
 	} else {
+		$acc .= 'void' if !$useHandle && $isVoid;
 		$acc .= "$proptype value" if $asProperty;
 		$acc .= $tok; # should be closing parenthesis
 	};
@@ -835,7 +842,7 @@ sub parse_parms
 	$acc =~ s/\(/,/;               # separating ( to ,
 	$acc =~ s/\)\;$//;             # eliminating ending );
 	$acc =~ s/\,\s*$//;            # eliminating tailing commas, if any
-	@chunks = split(",", $acc);
+	@chunks = grep { !/^\s*void$/ } split(",", $acc);
 	$acc  = "";
 	$hvUsed = 0;
 	my $i = 0;
@@ -1374,13 +1381,13 @@ sub out_method_profile
 		print HEADER "CALL_POINT:\n" if $property;
 		print HEADER "\n\tPUTBACK;\n\n";
 		if ( exists( $structs{ $resSub}) && ${$structs{ $resSub}[2]}{hash}) {
-		$retType = "G_SCALAR";
+			$retType = "G_SCALAR";
 		} elsif ( exists( $structs{ $resSub}) || exists( $arrays{ $resSub}) || $hvName) {
-		$retType = "G_ARRAY";
+			$retType = "G_ARRAY";
 		} elsif ( $resSub eq "void") {
-		$retType = "G_DISCARD";
+			$retType = "G_DISCARD";
 		} else {
-		$retType = "G_SCALAR";
+			$retType = "G_SCALAR";
 		};
 
 		$retType = "set ? G_DISCARD : $retType" if $property;
@@ -1974,8 +1981,10 @@ for ( sort { $structs{$a}-> [PROPS]-> {order} <=> $structs{$b}-> [PROPS]-> {orde
 	if ( ${$structs{$_}[PROPS]}{genh})
 	{
 		my @types = @{$structs{$_}-> [TYPES]};
-		my @ids = @{$structs{$_}-> [IDS]};
+		my @ids   = @{$structs{$_}-> [IDS]};
+		my @def   = @{$structs{$_}-> [DEFS]};
 		print HEADER "typedef struct _$_ {\n";
+		my ($maxw_undefs, @undefs) = (0);
 		for ( my $j = 0; $j < @types; $j++) {
 			if ( ref $types[$j]) {
 				print HEADER "\t$types[$j]->[PROPS]->{name} $ids[$j];\n";
@@ -1984,11 +1993,21 @@ for ( sort { $structs{$a}-> [PROPS]-> {order} <=> $structs{$b}-> [PROPS]-> {orde
 			} else {
 				print HEADER "\t$types[$j] $ids[$j];\n";
 			}
+
+			if (($def[$j] // '') =~ /^undef:/) {
+				push @undefs, $ids[$j];
+				$maxw_undefs = length $ids[$j] if length($ids[$j]) > $maxw_undefs;
+			}
+		}
+		if ( @undefs ) {
+			print HEADER "\tstruct {\n";
+			printf HEADER "\t\tunsigned %\-${maxw_undefs}s : 1;\n", $_ for @undefs;
+			print HEADER "\t} undef;\n";
 		}
 		print HEADER "} $_, *P$_;\n\n";
 		if ( ${$structs{$_}[PROPS]}{hash})
 		{
-			print HEADER "extern $_ * SvHV_$_( SV * hashRef, $_ * strucRef, char * errorAt);\n";
+			print HEADER "extern $_ * SvHV_$_( SV * hashRef, $_ * strucRef, const char * errorAt);\n";
 			print HEADER "extern SV * sv_${_}2HV( $_ * strucRef);\n";
 		}
 		print HEADER "extern $_ ${_}_buffer;\n\n";
@@ -2157,8 +2176,8 @@ SD
 	{
 		if ( ${$structs{$_}[2]}{genh} && ${$structs{$_}[2]}{hash})
 		{
-			print HEADER "$_ * SvHV_$_( SV * hashRef, $_ * strucRef, char * errorAt)\n{\n";
-			print HEADER "\tchar * err = errorAt ? errorAt : \"$_\";\n";
+			print HEADER "$_ * SvHV_$_( SV * hashRef, $_ * strucRef, const char * errorAt)\n{\n";
+			print HEADER "\tconst char * err = errorAt ? errorAt : \"$_\";\n";
 			print HEADER "\tHV * $incHV = ( HV*)\n\t".
 				"(( SvROK( hashRef) && ( SvTYPE( SvRV( hashRef)) == SVt_PVHV)) ? SvRV( hashRef)\n\t\t".
 				": ( croak( \"Illegal hash reference passed to %s\", err), nil));\n";
@@ -2186,9 +2205,13 @@ SD
 	}
 CONTAINED_STRUCTURE
 				} else {
-					$inter = "( $incSV = hv_fetch( $incHV, \"$lName\", $lNameLen, 0)) ? \n\t".
-						sv2type( $lType, "*$incSV")." : $def";
-					print HEADER "\t", cwrite( $lType, $inter, "strucRef-> $lName"), "\n";
+					print HEADER "\t$incSV = hv_fetch( $incHV, \"$lName\", $lNameLen, 0);\n";
+					if ($def =~ /^undef:(.*)$/) {
+						print HEADER "\tstrucRef-> undef.$lName = ($incSV == NULL);\n";
+						$def = $1;
+					}
+					$inter = "$incSV ? " . sv2type( $lType, "*$incSV") . " : $def";
+					print HEADER "\t", cwrite( $lType, $inter, "strucRef-> $lName"), "\n\n";
 				}
 			}
 			print HEADER "\treturn strucRef;\n";
@@ -2202,7 +2225,11 @@ CONTAINED_STRUCTURE
 				my $lNameLen = length $lName;
 				my $lType = @{$structs{$_}[TYPES]}[$k];
 				my $inter = type2sv( $lType, "strucRef-> $lName");
-				print HEADER "\t(void) hv_store( $incHV, \"$lName\", $lNameLen, $inter, 0);\n";
+				my $prefix = 
+					($structs{$_}->[DEFS]->[$k] =~ /^undef:/) ?
+						"if (!strucRef-> undef.$lName)" :
+						"(void)";
+				print HEADER "\t$prefix hv_store( $incHV, \"$lName\", $lNameLen, $inter, 0);\n";
 			}
 			print HEADER "\treturn newRV_noinc(( SV*) $incHV);\n";
 			print HEADER "}\n\n";
