@@ -97,14 +97,14 @@ sub init
 	my $self = shift;
 	unless ( @images) {
 		my $i = 0;
-		for ( sbmp::OutlineCollaps, sbmp::OutlineExpand) {
+		for ( sbmp::OutlineCollapse, sbmp::OutlineExpand) {
 			$images[ $i++] = Prima::StdBitmap::image($_);
 		}
 		if ( $images[0]) {
 			@imageSize = $images[0]-> size;
 		} else {
 			# if compiled without gif support
-			@imageSize = (11,11);
+			@imageSize = map { $::application->uiScaling * $_ } (11,11);
 			for my $i (0,1) {
 				$images[$i] = Prima::DeviceBitmap->new(
 					width     => $imageSize[0],
@@ -926,10 +926,9 @@ sub reset_scrolls
 	}
 }
 
-sub reset_tree
+sub reset_item_cache
 {
 	my ( $self, $i) = ( $_[0], 0);
-	$self-> makehint(0);
 	$self-> {stackFrames} = [];
 	$self-> {lineDefs}    = [];
 	my @stack;
@@ -965,6 +964,13 @@ sub reset_tree
 
 	$self-> {count} = $i;
 
+}
+
+sub reset_tree
+{
+	my $self = shift;
+	$self-> makehint(0);
+	$self-> reset_item_cache;
 	my $fullc = $self-> {fullCalibrate};
 	my ( $notifier, @notifyParms) = $self-> get_notify_sub(q(MeasureItem));
 	my $maxWidth = 0;
