@@ -1,28 +1,4 @@
 #
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
 #  Created by:
 #     Dmitry Karasik <dk@plab.ku.dk> 
 #  Modifications by:
@@ -30,8 +6,6 @@
 #  Documentation by:
 #     Anton Berezin  <tobez@tobez.org>
 #
-#  $Id$
-
 package Prima::ScrollBar;
 use vars qw(@ISA @stdMetrics);
 @ISA = qw(Prima::Widget Prima::MouseScroller);
@@ -40,6 +14,7 @@ use vars qw(@ISA @stdMetrics);
 my $win32 = (Prima::Application-> get_system_info-> {apc} == apc::Win32);
 
 use strict;
+use warnings;
 use Prima::Const;
 use Prima::IntUtils;
 
@@ -134,15 +109,61 @@ sub on_size
 	$_[0]-> reset;
 }
 
+sub draw_pad
+{
+	my ( $self, $canvas, $part, $base_color, $cache) = @_;
+
+	if ( defined $self->{prelight} && $self->{prelight} eq $part && $self->enabled ) {
+		$base_color = $self-> prelight_color($base_color, 1.25);
+	}
+
+	my @palette = ( $base_color, $self->dark3DColor );
+	my @spline  = ( 1, 0 );
+	unless ($self-> { vertical } ) {
+		@palette = reverse @palette;
+		@spline  = reverse @spline;
+	}
+
+	$self-> rect_bevel( $canvas, @{$self-> {$part}-> {rect}},
+		width   => 2, 
+		concave => $self->{$part}->{pressed},
+		fill    => $canvas->new_gradient(
+			palette   => \@palette,
+			spline    => \@spline,
+			vertical  => $self->{vertical},
+		),
+	);
+}
+
+sub fix_triangle
+{
+	my ( $v, $spot ) = @_;
+	if ( $v ) {
+		my $d = $$spot[4] - $$spot[2];
+		if ($d % 2) {
+			$$spot[0] = $$spot[2] + ($d - 1) / 2;
+			$$spot[2]--;
+		}
+	} else {
+		my $d = $$spot[3] - $$spot[5];
+		if ($d % 2) {
+			$$spot[1] = $$spot[5] + ($d - 1) / 2;
+			$$spot[5]--;
+		}
+	}
+}
+
+
 sub on_paint
 {
 	my ($self,$canvas) = @_;
 	my @clr;
 	my @c3d = ( $self-> light3DColor, $self-> dark3DColor);
-	if ( $self-> enabled)
-	{
+	if ( $self-> enabled) {
 		@clr = ($self-> color, $self-> backColor);
-	} else { @clr = ($self-> disabledColor, $self-> disabledBackColor); }
+	} else {
+		@clr = ($self-> disabledColor, $self-> disabledBackColor);
+	}
 	my @size = $canvas-> size;
 	my $btx  = $self-> { btx};
 	my $v    = $self-> { vertical};
@@ -154,14 +175,10 @@ sub on_paint
 	$canvas-> line( 0, $maxy, $maxx, $maxy);
 	$canvas-> line( 0, 0, 0, $maxy);
 	{
-		$self-> rect_bevel( $canvas, @{$self-> {b1}-> {rect}}, 
-			width   => 2, 
-			fill    => $clr[1],
-			concave => $self->{b1}->{pressed},
-		);
+		$self-> draw_pad( $canvas, b1 => $clr[1]);
 		$canvas-> color( $self-> {b1}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b1}-> { pressed} ? 1 : 0;
-		my @spot = $v ? (
+		my @spot = map { int($_ + .5) } $v ? (
 			$maxx * 0.5 + $a, $maxy - $btx * 0.40 - $a,
 			$maxx * 0.3 + $a, $maxy - $btx * 0.55 - $a,
 			$maxx * 0.7 + $a, $maxy - $btx * 0.55 - $a,
@@ -170,26 +187,26 @@ sub on_paint
 			$btx * 0.55 + $a, $maxy * 0.7 - $a,
 			$btx * 0.55 + $a, $maxy * 0.3 - $a
 		);
-		$canvas-> fillpoly( [ @spot]);
+		fix_triangle($v, \@spot);
+		$canvas-> fillpoly( [ @spot])
+			if $maxx > 10 && $maxy > 10;
 	}
 	{
-		$self-> rect_bevel( $canvas, @{$self-> {b2}-> {rect}}, 
-			width   => 2, 
-			fill    => $clr[1],
-			concave => $self->{b2}->{pressed},
-		);
+		$self-> draw_pad( $canvas, b2 => $clr[1]);
 		$canvas-> color( $self-> {b2}-> { enabled} ? $clr[ 0] : $self-> disabledColor);
 		my $a = $self-> { b2}-> { pressed} ? 1 : 0;
-		my @spot = $v ? (
-			$maxx * 0.5 + $a, $btx * 0.40 + 1 - $a,
-			$maxx * 0.3 + $a, $btx * 0.55 + 1 - $a,
-			$maxx * 0.7 + $a, $btx * 0.55 + 1 - $a
+		my @spot = map { int($_ + .5) } $v ? (
+			$maxx * 0.5 + $a, $btx * 0.40 - $a,
+			$maxx * 0.3 + $a, $btx * 0.55 - $a,
+			$maxx * 0.7 + $a, $btx * 0.55 - $a
 		) : (
-			$maxx - $btx * 0.45 + 1 + $a, $maxy * 0.5 - $a,
-			$maxx - $btx * 0.60 + 1 + $a, $maxy * 0.7 - $a,
-			$maxx - $btx * 0.60 + 1 + $a, $maxy * 0.3 - $a
+			$maxx - $btx * 0.40 + $a, $maxy * 0.5 - $a,
+			$maxx - $btx * 0.55 + $a, $maxy * 0.7 - $a,
+			$maxx - $btx * 0.55 + $a, $maxy * 0.3 - $a
 		);
-		$canvas-> fillpoly( [ @spot]);
+		fix_triangle($v, \@spot);
+		$canvas-> fillpoly( [ @spot])
+			if $maxx > 10 && $maxy > 10;
 		$canvas-> color( $clr[ 1]);
 	}
 	if ( $self-> { tab}-> { enabled})
@@ -217,13 +234,15 @@ sub on_paint
 			}
 		}
 
-		$self-> rect_bevel( $canvas, @rect,
-			width   => 2, 
-			fill    => $clr[1],
-			concave => $self->{tab}->{pressed},
-		);
-		if ( $self-> {minThumbSize} > 8 && $self->{style} ne 'xp')
+		$self-> draw_pad( $canvas, tab => $clr[1]);
+		if ( 
+			$self-> {minThumbSize} > 8 && 
+			$self->{style} ne 'xp' && 
+			(( $v ? $maxx : $maxy) > 10)
+		)
 		{
+			$canvas-> color( $clr[ 0]);
+			$canvas-> backColor( $c3d[ 0]);
 			if ( $v)
 			{
 				my $sty = $rect[1] + 
@@ -232,27 +251,21 @@ sub on_paint
 					( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $stx = int($maxx / 3) + ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lnx = int($maxx / 3);
-				$canvas-> color( $c3d[ 0]);
+				$lnx += $maxx - $lnx * 3;
+				$canvas-> fillPattern([(0xff, 0) x 4]);
+				$canvas-> fillPatternOffset($stx, $sty);
 				$canvas-> bar( $stx, $sty - 1, $stx + $lnx, $sty + 9);
-				$canvas-> color( $clr[ 0]);
-				$canvas-> lines( [ map { 
-					$stx,        $sty + $_ * 2, 
-					$stx + $lnx, $sty + $_ * 2 
-				} 0..5 ] );
 			} else {
 				my $stx = $rect[0] + 
 					int($lenx / 2) - 
-					6 + 
+					4 + 
 					( $self-> { tab}-> { pressed} ? 1 : 0) ;
 				my $sty = int($maxy / 3) - ( $self-> { tab} -> { pressed} ? 1 : 0);
 				my $lny = int($maxy / 3);
-				$canvas-> color( $c3d[ 0]);
-				$canvas-> bar( $stx + 1, $sty, $stx + 11, $sty + $lny);
-				$canvas-> color( $clr[ 0]);
-				$canvas-> lines( [ map { 
-					$stx + $_ * 2, $sty, 
-					$stx + $_ * 2, $sty + $lny
-				} 0..5 ] );
+				$lny += $maxy - $lny * 3;
+				$canvas-> fillPattern([(0xAA) x 8]);
+				$canvas-> fillPatternOffset($stx, $sty);
+				$canvas-> bar( $stx - 1, $sty, $stx + 9, $sty + $lny);
 			}
 		}
 	} else {
@@ -445,8 +458,22 @@ sub on_mousemove
 {
 	my ( $self, $mod, $x, $y) = @_;
 	$self-> clear_event;
-	return unless defined $self-> {mouseTransaction};
-	my $who   = $self-> {mouseTransaction};
+
+	my $who  = $self-> {mouseTransaction};
+	unless (defined $who) {
+		my $who = $self-> translate_point( $x, $y);
+		my $prelight;
+		if ( $self->enabled && (($who // '') =~ /^(b1|b2|tab)$/) && $self-> {$who}-> {enabled}) {
+			$prelight = $who;
+		} else {
+			$prelight = undef;
+		}
+		if (($prelight // '') ne ($self->{prelight} // '')) {
+			$self->{prelight} = $prelight;
+			$self->repaint;
+		}
+		return;
+	}
 
 	if ( $who eq q(tab)) {
 		my @groove = @{$self-> {groove}-> {rect}};
@@ -482,6 +509,12 @@ sub on_mousemove
 	}
 }
 
+sub on_mouseleave
+{
+	my $self = shift;
+	$self-> repaint if defined( delete $self->{prelight} );
+}
+
 sub on_mousewheel
 {
 	my ( $self, $mod, $x, $y, $z) = @_;
@@ -544,7 +577,7 @@ sub reset
 		@rect = $v ? (
 			1, $maxy - $btx - $lenx - $atx, $maxx - 1, $maxy - $btx - $atx
 		) : (
-			$btx + $atx, 1, $btx + $atx + $lenx - 1, $maxy - 1
+			$btx + $atx, 1, $btx + $atx + $lenx, $maxy - 1
 		);
 		$self-> set(
 			cursorPos  => [ $rect[0] + 1, $rect[1] + 1],
@@ -683,8 +716,6 @@ sub vertical     {($#_)?$_[0]-> set_vertical  ($_[1])                   : return
 
 1;
 
-__DATA__
-
 =head1 NAME
 
 Prima::ScrollBar - standard scroll bars class
