@@ -249,6 +249,7 @@ sub prf_types
 			holdGlyph imageScale )],
 		modalResult  => ['modalResult'],
 		icon    => ['image',],
+		text    => ['hotKey'],
 	);
 	$_[0]-> prf_types_add( $pt, \%de);
 	return $pt;
@@ -346,6 +347,7 @@ sub prf_types
 		valign  => ['valignment',],
 		bool    => [qw(autoWidth autoHeight showAccelChar showPartial wordWrap)],
 		Handle  => ['focusLink',],
+		text    => ['hotKey'],
 	);
 	$_[0]-> prf_types_add( $pt, \%de);
 	return $pt;
@@ -574,8 +576,8 @@ sub paint_exterior
 	$vs ||= $avs;
 	$canvas-> rect3d( 0,0,$sz[0]-1,$sz[1]-1,$bw,
 		$self-> dark3DColor,$self-> light3DColor,$self-> backColor);
-	my $sw = 12;
-	my $sl = 16;
+	my $sw = 12 * $::application-> uiScaling;
+	my $sl = 16 * $::application-> uiScaling;
 	$hs = $hs ? $sw : 0;
 	$vs = $vs ? $sw : 0;
 	my @r = ( $bw, $bw+$hs, $sz[0]-$bw-1-$vs,$sz[1]-$bw-1);
@@ -1263,7 +1265,7 @@ sub prf_types
 {
 	my $pt = $_[ 0]-> SUPER::prf_types;
 	my %de = (
-		bool         => ['vertical'],
+		bool         => [qw(vertical indeterminate)],
 		uiv          => [qw(min max value threshold indent)],
 		gaugeRelief  => ['relief'],
 	);
@@ -1708,7 +1710,7 @@ sub on_show
 
 sub on_hook
 {
-	my ( $self, $who, $prop, $old, $new) = @_;
+	my ( $self, $who, $prop, $old, $new, $widget) = @_;
 	if ( $prop eq 'name') {
 		return unless exists $self-> {list}-> {$old};
 		$self-> {list}-> {$new} = $self-> {list}-> {$old};
@@ -1890,15 +1892,9 @@ sub prf_adjust_default
 	);
 }
 
-sub act_profile
-{
-	my $self = $_[0];
-	return (
-		$self-> SUPER::act_profile,
-		onChildCreate  => '$_[3]-> origin( $_[3]-> left-$_[3]-> owner-> left, $_[3]-> bottom-$_[3]-> owner-> bottom);',
-	);
-}
+use constant OFFSET => 11;
 
+sub o_delta_aperture { OFFSET, OFFSET }
 
 sub on_paint
 {
@@ -1956,7 +1952,7 @@ sub on_paint
 		$canvas-> rect3d( 2, 2, $sz[0] - 3, $sz[1] - $mh, 1, @c3d);
 	}
 	$canvas-> linePattern( lp::Dash);
-	$canvas-> rectangle( 12, 12, $sz[0] - 17, $sz[1] - ($self-> prf('style') ? 48 : -4) - $mh);
+	$canvas-> rectangle( OFFSET+1, OFFSET+1, $sz[0] - 17, $sz[1] - ($self-> prf('style') ? 48 : -4) - $mh);
 	$canvas-> linePattern( lp::Solid);
 	$canvas-> translate(@tr);
 	$self-> common_paint( $canvas);
