@@ -36,10 +36,12 @@ use vars qw($inspector
 	$form
 	$fastLoad
 	$writeMode
+	$dpi
 );
 
 $fastLoad = 1;
 $writeMode = 0;
+$dpi = 0;
 
 my $openFileDlg;
 my $saveFileDlg;
@@ -161,8 +163,8 @@ sub profile_default
 	my $def = $_[ 0]-> SUPER::profile_default;
 	my %prf = (
 		text           => 'Object Inspector',
-		width          => 280,
-		height         => 350,
+		width          => 280 * $::application-> uiScaling,
+		height         => 350 * $::application-> uiScaling,
 		left           => 6,
 		sizeDontCare   => 0,
 		originDontCare => 0,
@@ -193,16 +195,17 @@ sub init
 		delegations => [qw(Change)],
 	);
 
+	my $step = $self-> font-> width * 25;
 	$self-> {monger} = $self-> insert( Notebook =>
 		origin  => [ 0, $fh],
-		size    => [ 100,  $sz[1] - $fh * 2],
+		size    => [ $step,  $sz[1] - $fh * 2],
 		growMode => gm::Client,
 		pageCount => 2,
 	);
 
 	$self-> {mtabs} = $self-> insert( Button =>
 		origin   => [ 0, 0],
-		size     => [ 100, $fh],
+		size     => [ $step, $fh],
 		text     => '~Events',
 		growMode => gm::Floor,
 		name     => 'MTabs',
@@ -212,14 +215,14 @@ sub init
 
 	$self-> {plist} = $self-> {monger}-> insert_to_page( 0, 'Prima::VB::OPropListViewer' =>
 		origin   => [ 0, 0],
-		size     => [ 100, $sz[1] - $fh * 2],
+		size     => [ $step, $sz[1] - $fh * 2],
 		name       => 'PList',
 		growMode   => gm::Client,
 	);
 
 	$self-> {elist} = $self-> {monger}-> insert_to_page( 1, 'Prima::VB::OPropListViewer' =>
 		origin   => [ 0, 0],
-		size     => [ 100, $sz[1] - $fh * 2],
+		size     => [ $step, $sz[1] - $fh * 2],
 		name       => 'EList',
 		growMode   => gm::Client,
 	);
@@ -227,7 +230,7 @@ sub init
 
 	$self-> insert( 'Prima::VB::Divider' =>
 		vertical => 1,
-		origin => [ 100, 0],
+		origin => [ $step, 0],
 		size   => [ 6, $sz[1] - $fh],
 		min    => 50,
 		max    => 50,
@@ -236,8 +239,8 @@ sub init
 	);
 
 	$self-> {panel} = $self-> insert( Notebook =>
-		origin    => [ 106, 0],
-		size      => [ $sz[0]-106, $sz[1] - $fh],
+		origin    => [ $step + 6, 0],
+		size      => [ $sz[0] - $step - 6, $sz[1] - $fh],
 		growMode  => gm::Right,
 		name      => 'Panel',
 		pageCount => 1,
@@ -1257,12 +1260,12 @@ sub profile_default
 		text           => $::application-> name,
 		width          => $::application-> width - 12,
 		left           => 6,
-		bottom         => $::application-> height - 106 -
+		bottom         => $::application-> height - $::application->uiScaling * 106 -
 			$::application-> get_system_value(sv::YTitleBar) - 
 			$::application-> get_system_value(sv::YMenu),
 		sizeDontCare   => 0,
 		originDontCare => 0,
-		height         => 100,
+		height         => 100 * $::application->uiScaling,
 		icon           => $VB::ico,
 		menuItems      => [
 			['~File' => [
@@ -1306,6 +1309,13 @@ sub profile_default
 				[],
 				['-runitem' => '~Run' => 'Ctrl+F9' => '^F9' => \&form_run ],
 				['-breakitem' => '~Break' => \&form_cancel ],
+				['~Emulate resolution' => [
+					["*dpi0" => '~Native' => \&dpi_toggle ],
+					[],
+					map {
+						["dpi$_" => "$_ PPI", \&dpi_toggle ]
+					} (48, 72, 96, 120, 144, 168, 192, 216, 240, 264, 288, 312)
+				]],
 			]],
 			[],
 			['~Help' => [
@@ -1332,53 +1342,68 @@ sub init
 	my %classes = %Prima::VB::CfgMaint::classes;
 	my @pages   = @Prima::VB::CfgMaint::pages;
 
+	my $s = $::application->uiScaling;
 	$self-> set(
-		sizeMin => [ 350, $self-> height],
+		sizeMin => [ $s*350, $self-> height],
 		sizeMax => [ 16384, $self-> height],
 	);
 
+	my ( $dx, $dy ) = map { $s * $_ } (26,30);
+
+	my @images = map {
+		my $i = Prima::Icon->new;
+		if ( my $f = Prima::Utils::find_image( 'VB::VB.gif')) {
+			if ( $i->load($f, index => $_)) {
+				$i-> ui_scale;
+			}
+		} else {
+			$i = undef;
+		}
+		$i
+	} 1..4;
+
 	$self-> {newbutton} = $self-> insert( SpeedButton =>
-		origin    => [ 4, $self-> height - 30],
-		size      => [ 26, 26],
+		origin    => [ 4, $self-> height - $dy],
+		size      => [ $dx, $dx],
 		hint      => 'New',
-		imageFile => Prima::Utils::find_image( 'VB::VB.gif').':1',
+		image     => $images[0],
 		glyphs    => 2,
 		onClick   => sub { $VB::main-> new; } ,
 	);
 
 	$self-> {openbutton} = $self-> insert( SpeedButton =>
-		origin    => [ 32, $self-> height - 30],
-		size      => [ 26, 26],
+		origin    => [ 4 + $dx, $self-> height - $dy],
+		size      => [ $dx, $dx],
 		hint      => 'Open',
-		imageFile => Prima::Utils::find_image( 'VB::VB.gif').':2',
+		image     => $images[1],
 		glyphs    => 2,
 		onClick   => sub { $VB::main-> open; } ,
 	);
 
 	$self-> {savebutton} = $self-> insert( SpeedButton =>
-		origin    => [ 60, $self-> height - 30],
-		size      => [ 26, 26],
+		origin    => [ 4 + $dx*2, $self-> height - $dy],
+		size      => [ $dx, $dx],
 		hint      => 'Save',
-		imageFile => Prima::Utils::find_image( 'VB::VB.gif').':3',
+		image     => $images[2],
 		glyphs    => 2,
 		onClick   => sub { $VB::main-> save; } ,
 	);
 
 	$self-> {runbutton} = $self-> insert( SpeedButton =>
-		origin    => [ 88, $self-> height - 30],
-		size      => [ 26, 26],
+		origin    => [ 4 + $dx*3, $self-> height - $dy],
+		size      => [ $dx, $dx],
 		hint      => 'Run',
-		imageFile => Prima::Utils::find_image( 'VB::VB.gif').':4',
+		image     => $images[3],
 		glyphs    => 2,
 		onClick   => sub { $VB::main-> form_run} ,
 	);
 
-
+	my $ts_x = 8 + $dx * 5;
 	$self-> {tabset} = $self-> insert( TabSet =>
-		left   => 150,
+		left   => $ts_x,
 		name   => 'TabSet',
 		top    => $self-> height,
-		width  => $self-> width - 150,
+		width  => $self-> width - $ts_x,
 		growMode => gm::Ceiling,
 		topMost  => 1,
 		tabs     => [ @pages],
@@ -1387,8 +1412,8 @@ sub init
 	);
 
 	$self-> {nb} = $self-> insert( Widget =>
-		origin => [ 150, 0],
-		size   => [$self-> width - 150, $self-> height - $self-> {tabset}-> height],
+		origin => [ $ts_x, 0],
+		size   => [$self-> width - $ts_x, $self-> height - $self-> {tabset}-> height],
 		growMode => gm::Client,
 		name    => 'TabbedNotebook',
 		onPaint => sub {
@@ -1400,8 +1425,8 @@ sub init
 	);
 
 	$self-> {nbpanel} = $self-> {nb}-> insert( Notebook =>
-		origin     => [12,1],
-		size       => [$self-> {nb}-> width-24,44],
+		origin     => [$s*12,1],
+		size       => [$self-> {nb}-> width-$s*24,36*$s+8],
 		growMode   => gm::Floor,
 		backColor  => cl::Gray,
 		name       => 'NBPanel',
@@ -1411,33 +1436,34 @@ sub init
 			$canvas-> rect3d(0,0,$sz[0]-1,$sz[1]-1,
 				1,cl::Black,cl::Black,$self-> backColor);
 			my $i = 0;
-			$canvas-> rectangle($i-38,2,$i,40) while (($i+=40)<($sz[0]+36));
+			my $sz = $s * 36;
+			$canvas-> rectangle($i-$sz-2,2,$i,$sz+4) while (($i+=$sz+4)<($sz[0]+$sz));
 		},
 	);
 
 	$self-> {leftScroll} = $self-> {nb}-> insert( SpeedButton =>
 		origin  => [1,5],
-		size    => [11,36],
+		size    => [map { $s * $_ } 11,36],
 		name    => 'LeftScroll',
 		autoRepeat => 1,
 		onPaint => sub {
 			$_[0]-> on_paint( $_[1]);
 			$_[1]-> color( $_[0]-> enabled ? cl::Black : cl::Gray);
-			$_[1]-> fillpoly([7,4,7,32,3,17]);
+			$_[1]-> fillpoly([map { $s * $_ } 7,4,7,32,3,17]);
 		},
 		delegations => [ $self, qw(Click)],
 	);
 
 	$self-> {rightScroll} = $self-> {nb}-> insert( SpeedButton =>
-		origin  => [$self-> {nb}-> width-11,5],
-		size    => [11,36],
+		origin  => [$self-> {nb}-> width-11*$s,5],
+		size    => [map { $s * $_ } 11,36],
 		name    => 'RightScroll',
 		growMode => gm::Right,
 		autoRepeat => 1,
 		onPaint => sub {
 			$_[0]-> on_paint( $_[1]);
 			$_[1]-> color( $_[0]-> enabled ? cl::Black : cl::Gray);
-			$_[1]-> fillpoly([3,4,3,32,7,17]);
+			$_[1]-> fillpoly([map { $s * $_ } 3,4,3,32,7,17]);
 		},
 		delegations => [ $self, qw(Click)],
 	);
@@ -1540,7 +1566,8 @@ sub reset_tabs
 
 	my %iconfails = ();
 	my %icongtx   = ();
-	for ( keys %{$self-> {classes}}) {
+	my $s = $::application->uiScaling;
+	for ( sort keys %{$self-> {classes}}) {
 		my ( $class, %info) = ( $_, %{$self-> {classes}-> {$_}});
 		$offsets{$info{page}} = 4 unless exists $offsets{$info{page}};
 		next unless $modules{$info{module}};
@@ -1554,18 +1581,19 @@ sub reset_tabs
 				$iconfails{$info{icon}} = 1;
 				$i = undef;
 			}
+			$i-> ui_scale if $i;
 		};
 		my $j = $nb-> insert_to_page( $pagofs{$info{page}}, SpeedButton =>
 			hint   => $class,
 			name   => 'ClassSelector',
 			image  => $i,
 			origin => [ $offsets{$info{page}}, 4],
-			size   => [ 36, 36],
+			size   => [ map { $s * $_ } 36, 36],
 			delegations => [$self, qw(Click)],
 		);
 		$j-> {orgLeft}   = $offsets{$info{page}};
 		$j-> {className} = $class;
-		$offsets{$info{page}} += 40;
+		$offsets{$info{page}} += 36 * $s + 4;
 	}
 	$self-> {nbIndex} = 0;
 	$nb-> unlock;
@@ -1792,10 +1820,13 @@ sub push_widgets
 					$dep{$_}-> {profile}-> {owner} = $main;
 				}
 				my @ndelta = $owners{$_} eq $main ? (0,0) : (
-				$widgets{$owners{$_}}-> left,
-				$widgets{$owners{$_}}-> bottom
+					$widgets{$owners{$_}}-> left,
+					$widgets{$owners{$_}}-> bottom
 				);
-				$norg[$_] += $ndelta[$_] for 0..1;
+				my @aperture = $owners{$_} eq $main ? (0,0) : (
+					$widgets{$owners{$_}}-> o_delta_aperture
+				);
+				$norg[$_] += $ndelta[$_] + $aperture[$_] for 0..1;
 				$dep{$_}-> {profile}-> {origin} = \@norg;
 			}
 			$c-> prf_set( %{$dep{$_}-> {profile}});
@@ -1869,8 +1900,7 @@ sub load_file
 	$VB::inspector-> {selectorChanging} = 1 if $VB::inspector;
 	my $loaded;
 	$self-> push_widgets( sub {
-	$loaded++;
-	$self-> text( sprintf( "Loaded %d%%", ($loaded / $maxwij) * 100)); 
+		$self-> text( sprintf( "Loaded %d%%", ($loaded++ / $maxwij) * 100)); 
 	}, @seq);
 	$VB::form-> show;
 	$VB::inspector-> {selectorChanging}-- if $VB::inspector;
@@ -1937,34 +1967,34 @@ MEDI
 		my %extras    = $_-> ext_profile;
 		if ( scalar keys %extras) {
 			$c .= "\t\textras => {\n";
-			for ( keys %extras) {
+			for ( sort keys %extras) {
 				my $val  = $extras{$_};
 				my $type = $self-> get_typerec( $types-> {$_}, \$val);
 				$val = defined($val) ? $type-> write( $_, $val) : 'undef';
-				$c .= "\t\t$_ => $val,\n";
+				$c .= "\t\t\t$_ => $val,\n";
 			}
 			$c .= "\t\t},\n";
 		}
 		%extras    = $_-> act_profile;
 		if ( scalar keys %extras) {
 			$c .= "\t\tactions => {\n";
-			for ( keys %extras) {
+			for ( sort keys %extras) {
 				my $val  = $extras{$_};
 				my $type = $self-> get_typerec( $types-> {$_}, \$val);
 				$val = defined($val) ? $type-> write( $_, $val) : 'undef';
-				$c .= "\t\t$_ => $val,\n";
+				$c .= "\t\t\t$_ => $val,\n";
 			}
 			$c .= "\t\t},\n";
 		}
 		my %Handle_props = map { $_ => 1 } $_-> {prf_types}-> {Handle} ? @{$_-> {prf_types}-> {Handle}} : ();
 		delete $Handle_props{owner};
 		if ( scalar keys %Handle_props) {
-			$c .= "\t\tsiblings => [qw(" . join(' ', keys %Handle_props) . ")],\n";
+			$c .= "\t\tsiblings => [qw(" . join(' ', sort keys %Handle_props) . ")],\n";
 		}
 		$c .= "\t\tprofile => {\n";
 		my ( $x,$prf) = ($_, $_-> {profile});
 		my @o = $_-> get_o_delta;
-		for( keys %{$prf}) {
+		for( sort keys %{$prf}) {
 			my $val = $prf-> {$_};
 			if ( $_ eq 'origin' && defined $val) {
 				my @nval = (
@@ -2021,7 +2051,7 @@ sub profile_default
 PREHEAD
 	my $prf   = $VB::form-> {profile};
 	my $types = $VB::form-> {types};
-	for ( keys %$prf) {
+	for ( sort keys %$prf) {
 		my $val = $prf-> {$_};
 		my $type = $self-> get_typerec( $types-> {$_}, \$val);
 		$val = defined($val) ? $type-> write( $_, $val) : 'undef';
@@ -2075,7 +2105,7 @@ HEAD3
 		my %extras = $obj-> ext_profile;
 		next unless scalar keys %extras;
 		$c .= "\t\$instances{$_}->{extras} = {\n";
-		for ( keys %extras) {
+		for ( sort keys %extras) {
 			my $val  = $extras{$_};
 			my $type = $self-> get_typerec( $types-> {$_}, \$val);
 			$val = defined($val) ? $type-> write( $_, $val) : 'undef';
@@ -2085,7 +2115,7 @@ HEAD3
 	}
 
 	$c .= "\t".$actions{onBegin}->{$_}."(q($_), \$instances{$_});\n"
-		for keys %{$actions{onBegin}};
+		for sort keys %{$actions{onBegin}};
 	$c .= <<HEAD4;
 	my \%profile = \$self-> SUPER::init(\@_);
 	my \%names   = ( q($main) => \$self);
@@ -2098,7 +2128,7 @@ HEAD4
 	my @re_cmp = ();
 
 	$c .= "\t".$actions{onFormCreate}->{$_}."(q($_), \$instances{$_}, \$self);\n"
-		for keys %{$actions{onFormCreate}};
+		for sort keys %{$actions{onFormCreate}};
 	$c .= "\t".$actions{onCreate}->{$main}."(q($main), \$instances{$main}, \$self);\n"
 		if $actions{onCreate}->{$main};
 AGAIN:
@@ -2121,7 +2151,7 @@ AGAIN:
 		$c .= "\t\$names{$name} = \$names{$owner}-> insert( qq($class) => \n";
 		my ( $x,$prf) = ($_, $_-> {profile});
 		my @o = $_-> get_o_delta;
-		for ( keys %{$prf}) {
+		for ( sort keys %{$prf}) {
 			my $val = $prf-> {$_};
 			if ( $_ eq 'origin' && defined $val) {
 				my @nval = (
@@ -2152,7 +2182,7 @@ AGAIN:
 	}
 
 	$c .= "\t".$actions{onEnd}-> {$_}."(q($_), \$instances{$_}, \$names{$_});\n"
-		for keys %{$actions{onEnd}};
+		for sort keys %{$actions{onEnd}};
 $c .= <<POSTHEAD;
 	\$self-> unlock;
 	return \%profile;
@@ -2280,16 +2310,33 @@ sub form_run
 	my $okCreate = 0;
 	$VB::main-> {topLevel} = { map { ("$_" => 1) } $::application-> get_components };
 	@Prima::VB::VBLoader::eventContext = ('', '');
+	my $uiScaling = $::application->uiScaling;
+	my @saved_font = @Prima::Widget::default_font_box;
 	eval{
 		local $SIG{__WARN__} = sub { 
 			return if $_[0] =~ /^Subroutine.*redefined/;
 			die $_[0] 
 		};
+		my $n = $VB::form-> prf('name');
+		my %param;
+		if ( $VB::dpi > 0 ) {
+			my $sc = $VB::dpi / 96.0;
+			$::application->uiScaling($sc);
+			my %emulated_font;
+			%emulated_font = %{ $::application->get_default_font };
+			$emulated_font{$_} *= $sc for qw(size height width);
+			$param{$n}->{font} = \%emulated_font;
+			unless ( @Prima::Widget::default_font_box ) {
+				my $f = $::application-> get_default_font;
+				@Prima::Widget::default_font_box = ( $f-> { width}, $f-> { height});
+			}
+			$Prima::Widget::default_font_box[$_] *= $sc for 0,1;
+		}
 		my $sub = eval("$c");
 		die "Error loading module $@" if $@;
 		my @d = $sub-> ();
-		my %r = Prima::VB::VBLoader::AUTOFORM_REALIZE( \@d, {});
-		my $f = $r{$VB::form-> prf('name')};
+		my %r = Prima::VB::VBLoader::AUTOFORM_REALIZE( \@d, \%param );
+		my $f = $r{$n};
 		$okCreate = 1;
 		if ( $f) {
 			$f-> set( onClose => \&form_cancel );
@@ -2301,8 +2348,13 @@ sub form_run
 			$VB::editor-> hide if $VB::editor;
 		};
 	};
-	if ( $@) {
-		my $msg = "$@";
+	my $error = $@;
+	if ( $VB::dpi > 0 ) {
+		$::application->uiScaling($uiScaling);
+		@Prima::Widget::default_font_box = @saved_font;
+	}
+	if ( $error ) {
+		my $msg = "$error";
 		$msg =~ s/ \(eval \d+\)//g;
 		if ( defined( $Prima::VB::VBLoader::eventContext[0]) && 
 			length ($Prima::VB::VBLoader::eventContext[0])) {
@@ -2329,6 +2381,17 @@ sub form_run
 	}
 }
 
+sub dpi_toggle
+{
+	my $old = $VB::dpi;
+	my ( $self, $id ) = @_;
+	$id =~ s/^dpi//;
+	return if $id eq $old;
+	$self->menu->uncheck("dpi$old");
+	$self->menu->check("dpi$id");
+	$VB::dpi = $id;
+}
+
 sub wait
 {
 	my $t = $_[0]-> insert( Timer => timeout => 10, onTick => sub {
@@ -2477,12 +2540,12 @@ Visual Builder is a RAD-style suite for designing forms under
 the Prima toolkit. It provides rich set of perl-composed widgets,
 whose can be inserted into a form by simple actions. The form
 can be stored in a file and loaded by either user program or
-a simple wrapper, C<utils/fmview.pl>; the form can be also stored as
+a simple wrapper, C<utils/prima-fmview.pl>; the form can be also stored as
 a valid perl program. 
 
 A form file typically has I<.fm> extension, an can be loaded
 by using L<Prima::VB::VBLoader> module. The following example
-is the only content of C<fmview.pl>:
+is the only content of C<prima-fmview.pl>:
 
 	use Prima qw(Application VB::VBLoader);
 	my $ret = Prima::VBLoad( $ARGV[0] );
@@ -2670,7 +2733,7 @@ the form window grid granularity instead of the pixel granularity.
 =item Run
 
 This command hides the form and object inspector windows and
-'executes' the form, as if it would be run by C<fmview.pl>.
+'executes' the form, as if it would be run by C<prima-fmview.pl>.
 The execution session ends either by closing the form window
 or by calling L<Break> command.
 
