@@ -27,11 +27,8 @@ use strict;
 use warnings;
 use Carp;
 use Prima;
-use Prima::Classes;
 use Prima::Application name => "Font Dialog";
-use Prima::Lists;
-use Prima::Sliders;
-use Prima::Buttons;
+use Prima qw(Lists Sliders Buttons FileDialog);
 use Encode;
 
 # try to use perl5.8 glyph names
@@ -151,109 +148,13 @@ my $re_size = sub {
 $w = Prima::MainWindow-> create( text => "Font Window",
 	origin => [ 200, 200],
 	size   => [ 500, 530],
+	designScale => [7,16],
 	borderStyle => bs::Dialog,
 );
 
-$displayRes = ($w-> resolution)[1];
-for ( sort { $a-> {name} cmp $b-> {name}} @{$::application-> fonts})
+sub create_info_window
 {
-	$fontList{$_-> {name}} = $_;
-	push ( @fontItems, $_-> {name});
-}
-
-$w-> insert( ListBox =>
-	name   => "NameList",
-	origin => [25, 25],
-	size   => [ 225, 315],
-	items => [@fontItems],
-	onSelectItem => sub {
-		&$re_size(1);
-		&$re_sample;
-	},
-);
-
-$w-> insert( ListBox =>
-	name   => 'SizeList',
-	origin => [ 270, 230],
-	size   => [ 200, 110],
-	onSelectItem => sub {
-		$lastSizeSel = $_[0]-> get_item_text( $_[0]-> focusedItem);
-		&$re_sample;
-	},
-);
-
-$w-> insert( ListBox => 
-	origin      => [ 270, 160],
-	size        => [ 200, 55],
-	name        => 'Encoding',
-	onSelectItem => sub {
-		$lastEncSel = $_[0]-> get_item_text( $_[0]-> focusedItem);
-		&$re_size(0);
-		&$re_sample;
-	},
-	
-);
-
-$w-> insert( Button =>
-origin => [ 24, 348],
-size   => [ 32, 32],
-text   => 'B',
-name   => 'Bold',
-selectable => 0,
-font   => {
-	height => 20,
-	style  => fs::Bold,
-},
-checkable => 1,
-onClick   => sub {
-	$fs = ( $fs & fs::Bold ? $fs & ~fs::Bold : $fs | fs::Bold);
-	&$re_sample;
-},
-);
-
-$w-> insert( Button =>
-origin => [ 60, 348],
-size   => [ 32, 32],
-text   => 'I',
-name   => 'Italic',
-selectable => 0,
-font   => {
-	height => 20,
-	style  => fs::Italic,
-},
-checkable => 1,
-onClick   => sub {
-	$fs = (( $fs & fs::Italic) ? ($fs & ~fs::Italic) : ($fs | fs::Italic));
-	&$re_sample;
-},
-);
-
-$w-> insert( Button =>
-origin => [ 96, 348],
-size   => [ 32, 32],
-text   => 'U',
-selectable => 0,
-name   => 'Underlined',
-font   => {
-	height => 20,
-	style  => fs::Underlined,
-},
-checkable => 1,
-onClick   => sub {
-	$fs = (( $fs & fs::Underlined) ? ($fs & ~fs::Underlined) : ($fs | fs::Underlined));
-	&$re_sample;
-},
-);
-
-$w-> insert( Button =>
-origin => [ 142, 348],
-size   => [ 32, 32],
-text   => 'i',
-selectable => 0,
-name   => 'Info',
-color  => cl::Blue,
-font   => { height => 28, style => fs::Bold, name => "Tms Rmn"},
-onClick   => sub {
+	my $w = shift;
 	my $f = $w-> Example-> font;
 	my $ww = Prima::Window-> create(
 		size => [ 500, $f-> height * 3 + $f-> externalLeading + $f-> descent + 450 ],
@@ -345,6 +246,56 @@ onClick   => sub {
 			$p-> text_out( 'face name           : '.$cachedFacename, 2, $sd); $sd -= $fh;
 		},
 	);
+
+	my %wf = ( %$f, direction => 0 );
+	my @gsize= ( $f->maximalWidth + 10 , $f->height + 10 );
+	my $glyph = $ww-> insert( Widget => 
+		text => ' ',
+		font => \%wf,
+		size => \@gsize,
+		origin => [ $ww-> width - $gsize[0] - 10, $ww-> height - $gsize[1] - 10 ],
+		growMode => gm::GrowLoX|gm::GrowLoY,
+		backColor => cl::White,
+		#buffered => 1,
+		name    => 'Glyph',
+		textOutBaseline => 0,
+		onPaint => sub {
+			my $self = shift;
+			$self-> clear;
+			my $C = $self-> text;
+
+			my ( $a, $b, $c ) = @{ $self->get_font_abc( ord($C), ord($C), utf8::is_utf8($C)) };
+			my ( $d, $e, $f ) = @{ $self->get_font_def( ord($C), ord($C), utf8::is_utf8($C)) };
+			
+			my $w = (( $a < 0 ) ? 0 : $a) + $b + (( $c < 0 ) ? 0 : $c);
+			my $h = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f);
+			$w = ( $self-> width  - $w ) / 2;
+			$h = ( $self-> height - $h ) / 2;
+			$self-> translate($w, $h);
+
+			my $dx = 0;
+			my $dy = 0;
+			$dx -= $a if $a < 0;
+			$dy -= $d if $d < 0;
+
+			my $fh = $self-> font->height;
+			$self-> text_out( $C, $dx, $dy );
+
+			$dx = abs($a);
+			$dy = abs($d);
+			$self-> linePattern(lp::Dot);
+			$self-> line($dx, 0, $dx, $self->height);
+			$dx = (( $a < 0 ) ? 0 : $a) + $b + (( $c < 0 ) ? 0 : $c) - abs($c);
+			$self-> line($dx, 0, $dx, $self->height);
+			$self-> line(-$w, $self->font->descent, $self->width, $self->font->descent);
+
+			$self-> line(0, $dy, $self->width, $dy);
+			$dy = (( $d < 0 ) ? 0 : $d) + $e + (( $f < 0 ) ? 0 : $f) - abs($f);
+			$self-> line(0, $dy, $self->width, $dy);
+		},
+	);
+
+
 	my @ranges = ([]);
 	for ( @{$w-> Example-> get_font_ranges}) {
 		( 2 > scalar @{$ranges[-1]}) ?
@@ -378,6 +329,8 @@ onClick   => sub {
 				}
 				$self-> hint( $pretty );
 				$self-> hintVisible(1);
+				$glyph->text(chr($c));
+				$glyph->repaint;
 			}
 		},
 		onDrawItem => sub {
@@ -400,7 +353,141 @@ onClick   => sub {
 	);
 	$l-> count( $count);
 	$ww-> select;
-},
+}
+
+$displayRes = ($w-> resolution)[1];
+
+my $load_fonts = sub {
+	%fontList = ();
+	@fontItems = ();
+	for ( sort { $a-> {name} cmp $b-> {name}} @{$::application-> fonts})
+	{
+		$fontList{$_-> {name}} = $_;
+		push ( @fontItems, $_-> {name});
+	}
+};
+
+$load_fonts->();
+
+$w-> insert( ListBox =>
+	name   => "NameList",
+	origin => [25, 25],
+	size   => [ 225, 315],
+	items => [@fontItems],
+	onSelectItem => sub {
+		&$re_size(1);
+		&$re_sample;
+	},
+);
+
+$w-> insert( ListBox =>
+	name   => 'SizeList',
+	origin => [ 270, 230],
+	size   => [ 200, 110],
+	onSelectItem => sub {
+		$lastSizeSel = $_[0]-> get_item_text( $_[0]-> focusedItem);
+		&$re_sample;
+	},
+);
+
+$w-> insert( ListBox => 
+	origin      => [ 270, 160],
+	size        => [ 200, 55],
+	name        => 'Encoding',
+	onSelectItem => sub {
+		$lastEncSel = $_[0]-> get_item_text( $_[0]-> focusedItem);
+		&$re_size(0);
+		&$re_sample;
+	},
+	
+);
+
+my $scaling = $::application->font->height / ($w->designScale)[1];
+
+$w-> insert( Button =>
+	origin => [ 24, 348],
+	size   => [ 32, 32],
+	text   => 'B',
+	name   => 'Bold',
+	selectable => 0,
+	font   => {
+		height => 20 * $scaling,
+		style  => fs::Bold,
+	},
+	checkable => 1,
+	onClick   => sub {
+		$fs = ( $fs & fs::Bold ? $fs & ~fs::Bold : $fs | fs::Bold);
+		&$re_sample;
+	},
+);
+
+$w-> insert( Button =>
+	origin => [ 60, 348],
+	size   => [ 32, 32],
+	text   => 'I',
+	name   => 'Italic',
+	selectable => 0,
+	font   => {
+		height => 20 * $scaling,
+		style  => fs::Italic,
+	},
+	checkable => 1,
+	onClick   => sub {
+		$fs = (( $fs & fs::Italic) ? ($fs & ~fs::Italic) : ($fs | fs::Italic));
+		&$re_sample;
+	},
+);
+
+$w-> insert( Button =>
+	origin => [ 96, 348],
+	size   => [ 32, 32],
+	text   => 'U',
+	selectable => 0,
+	name   => 'Underlined',
+	font   => {
+		height => 20 * $scaling,
+		style  => fs::Underlined,
+	},
+	checkable => 1,
+	onClick   => sub {
+		$fs = (( $fs & fs::Underlined) ? ($fs & ~fs::Underlined) : ($fs | fs::Underlined));
+		&$re_sample;
+	},
+);
+
+$w-> insert( Button =>
+	origin => [ 142, 348],
+	size   => [ 32, 32],
+	text   => 'i',
+	selectable => 0,
+	name   => 'Info',
+	color  => cl::Blue,
+	font   => { height => 28 * $scaling, style => fs::Bold },
+	onClick   => sub { create_info_window($w) },
+);
+
+$w-> insert( Button =>
+	origin => [ 180, 348],
+	size   => [ 32, 32],
+	text   => '+',
+	hint   => 'Add new font',
+	selectable => 0,
+	name   => 'Load',
+	font   => { height => 20 * $scaling },
+	onClick   => sub {
+		my $d = Prima::FileDialog-> new;
+		my $f = $d->execute;
+		return unless defined $f;
+		my $error = '';
+		local $SIG{__WARN__} = sub { $error = ": @_" };
+		unless ($::application->load_font($d->fileName)) {
+			$error =~ s/at examples.*//;
+			return Prima::message("Error loading font$error");
+		}
+	
+		$load_fonts->();
+		$w->NameList->items(\@fontItems);
+	}
 );
 
 
