@@ -5,7 +5,7 @@
 # XXX backport tb's to TextView
 use strict;
 use warnings;
-use Prima qw(Application MsgBox Buttons Edit Notebooks Label DetailedList Outlines Drawable::Markup);
+use Prima qw(Application Buttons Edit Notebooks Label DetailedList Outlines Drawable::Markup MsgBox);
 use FindBin qw($Bin);
 
 my $fp = [
@@ -23,6 +23,7 @@ my $Main = Prima::MainWindow->create(
 	name   => 'Main',
 	text   => 'Markup test',
 	size   => [500,500],
+	designScale => [7, 16],
 );
 
 my $tn = $Main->insert('TabbedNotebook',
