@@ -1,31 +1,3 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  $Id$
-#
-
 =pod 
 
 =head1 NAME
@@ -40,13 +12,16 @@ Note the custom pointer creation and its dynamic change ( the "User" button ).
 =cut
 
 use strict;
+use warnings;
 use Prima qw( StdBitmap Buttons Application);
 
 package UserInit;
 
 my $ph = Prima::Application-> get_system_value(sv::YPointer);
+my $sc = $::application-> uiScaling;
+$ph = $::application->font->height * 2 if $ph < $::application->font->height * 2;
 my $w = Prima::MainWindow-> create(
-	size    => [ 350, 20 + ($ph+8)*11],
+	size    => [ 350 * $sc, 20 + ($ph+8)*11],
 	left    => 200,
 	text    => 'Pointers',
 );
@@ -69,8 +44,8 @@ for my $c ( @a[0..$#a-1])
 	$w-> pointer( $p);
 	my $b = $w-> insert( Button =>
 		flat => 1,
-		left    => 10 + (($i-1) % 2)*170,
-		width   => 160,
+		left    => (10 + (($i-1) % 2)*170) * $sc,
+		width   => 160 * $sc,
 		height  => $ph + 4,
 		bottom  => $w-> height - int(($i+1)/2) * ($ph+8) - 10,
 		pointer => $p,
@@ -81,7 +56,7 @@ for my $c ( @a[0..$#a-1])
 	$i++;
 };
 
-my $ptr = Prima::StdBitmap::icon( sbmp::DriveCDROM);
+my $ptr = Prima::StdBitmap::icon( sbmp::DriveCDROM, argb => 0, copy => 1);
 
 my @mapset = map {
 	my ($x,$a) = $ptr-> split;
@@ -98,8 +73,8 @@ my @mapset = map {
 my $mapsetID = 0;
 
 my $b = $w-> insert( SpeedButton =>
-	left    => 10 + (($i-1) % 2)*170,
-	width   => 160,
+	left    => $sc * ( 10 + (($i-1) % 2)*170 ),
+	width   => 160 * $sc,
 	height  => $ph+4,
 	bottom  => $w-> height - int(($i+1)/2) * ($ph+8) - 10,
 	pointer => $ptr,
