@@ -1,31 +1,3 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  $Id$
-#
-
 =pod 
 
 =head1 NAME
@@ -73,6 +45,7 @@ my $w = MyWindow-> create(
 	bottom  => 100,
 	width   => 300,
 	height  => 300,
+	designScale => [7, 16],
 	borderStyle => bs::Sizeable,
 );
 
@@ -82,7 +55,7 @@ $w-> insert(
 	 origin   => [ 2, 20],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 $w-> insert(
@@ -91,7 +64,7 @@ $w-> insert(
 	 origin   => [ 2, 40],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 $w-> insert(
@@ -100,7 +73,7 @@ $w-> insert(
 	 origin   => [ 2, 60],
 	 vertical => 0,
 	 max      => 255,
-	 width    => $w-> width - 2,
+	 width    => 298,
 	 onChange => sub { $w-> updateArea },
 );
 
@@ -110,7 +83,7 @@ $w-> insert(
 	 origin   => [ 2, 80],
 	 vertical => 1,
 	 max      => 255,
-	 height    => $w-> height - 80,
+	 height    => 218,
 );
 
 
