@@ -1,36 +1,10 @@
-#
-#  Copyright (c) 1997-2002 The Protein Laboratory, University of Copenhagen
-#  All rights reserved.
-#
-#  Redistribution and use in source and binary forms, with or without
-#  modification, are permitted provided that the following conditions
-#  are met:
-#  1. Redistributions of source code must retain the above copyright
-#     notice, this list of conditions and the following disclaimer.
-#  2. Redistributions in binary form must reproduce the above copyright
-#     notice, this list of conditions and the following disclaimer in the
-#     documentation and/or other materials provided with the distribution.
-#
-#  THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
-#  ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
-#  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
-#  ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
-#  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
-#  DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
-#  OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
-#  HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
-#  LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
-#  OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
-#  SUCH DAMAGE.
-#
-#  $Id$
-#
 =pod 
-=item NAME
 
-Prima toolkit example
+=head1 NAME
 
-=item FEATURES
+examples/transparent.pl - Prima toolkit example
+
+=head1 FEATURES
 
 Demonstrates the usage of Prima::Widget::transparent property.
 Test the certain events: for example, when two transparent
@@ -41,11 +15,15 @@ Note that the $tt widget is not transparent but uses shape extension.
 
 =cut
 
+use strict;
+use warnings;
+
 use Prima;
 use Prima::Const;
 use Prima::Application name => 'Generic.pm';
 
-$w = Prima::MainWindow-> create(
+
+my $w = Prima::MainWindow-> create(
 	size => [ 300, 300],
 	borderStyle => bs::Dialog,
 	backColor => cl::Green,
@@ -93,14 +71,11 @@ $w-> insert(
 );
 
 
+goto NOSHAPE unless $::application-> get_system_value( sv::ShapeExtension);
 my $string = "Hello from Prima::OnScreenDisplay!";
 my $tt = Prima::Widget-> create(
 	name => 'W1',
-	onPaint => sub {
-		$_[1]-> color( cl::LightRed);
-		$_[1]-> font-> size( 36);
-		$_[1]-> text_out( $string, 0, 0);
-	},
+	backColor => cl::LightRed,
 	onMouseDown => sub {
 		$_[0]-> {drag}    = [ $_[3], $_[4]];
 		$_[0]-> {lastPos} = [ $_[0]-> left, $_[0]-> bottom];
@@ -121,7 +96,7 @@ my $tt = Prima::Widget-> create(
 );
 
 $tt-> begin_paint_info;
-$tt-> font-> size(36);
+$tt-> font-> height( $tt-> height * 0.7 );
 my $font = $tt-> font;
 $tt-> width( $tt-> get_text_width( $string));
 $tt-> end_paint_info;
@@ -137,5 +112,6 @@ $i-> text_out( $string, 0, 0);
 $i-> end_paint;
 $tt-> shape($i);
 $tt-> bring_to_front;
+NOSHAPE:
 
 run Prima;
