@@ -833,14 +833,20 @@ static Bool
 save( PImgCodec instance, PImgSaveFileInstance fi)
 {
 	dPROFILE;
-	PIcon i = ( PIcon) fi-> object;
-	SaveRec * l = ( SaveRec *) fi-> instance;
-	HV * profile = fi-> objectExtras;
+	PIcon i;
+	SaveRec * l;
+	HV * profile;
 	Bool icon;
-	Byte * alpha = NULL;
+	Byte * alpha;
 	
+	l = ( SaveRec *) fi-> instance;
 	if ( setjmp( png_jmpbuf( l-> png_ptr)) != 0) return false;
+
+	l = ( SaveRec *) fi-> instance;
 	icon = kind_of( fi-> object, CIcon);
+	i = ( PIcon) fi-> object;
+	profile = fi-> objectExtras;
+	alpha = NULL;
 
 	/* header */
 	{
