@@ -136,7 +136,7 @@ Application_cleanup( Handle self)
 		my-> detach( self, var-> icon, true);
 	var-> icon = nilHandle;
 
-	my-> first_that_component( self, (void*)kill_all, nil);
+	my-> first_that_component( self, (void*)prima_kill_all_objects, nil);
 
 	CDrawable-> cleanup( self);
 }
@@ -576,7 +576,7 @@ Application_colorIndex( Handle self, Bool set, int index, Color color)
 		SingleColor s;
 		s. color = color;
 		s. index = index;
-		if ( !opt_InPaint) my-> first_that( self, (void*)single_color_notify, &s);
+		if ( !opt_InPaint) my-> first_that( self, (void*)prima_single_color_notify, &s);
 		if ( opt_InPaint) switch ( index) {
 			case ciFore:
 				CDrawable-> set_color ( self, color);
@@ -593,7 +593,7 @@ Application_colorIndex( Handle self, Bool set, int index, Color color)
 void
 Application_set_font( Handle self, Font font)
 {
-	if ( !opt_InPaint) my-> first_that( self, (void*)font_notify, &font);
+	if ( !opt_InPaint) my-> first_that( self, (void*)prima_font_notify, &font);
 	apc_font_pick( self, &font, & var-> font);
 	if ( opt_InPaint) apc_gp_set_font ( self, &var-> font);
 }
@@ -1025,7 +1025,8 @@ Bool   Application_ownerBackColor( Handle self, Bool set, Bool ownerBackColor) {
 Bool   Application_ownerFont( Handle self, Bool set, Bool ownerFont)   { return false; }
 Bool   Application_ownerShowHint( Handle self, Bool set, Bool ownerShowHint) { return false; }
 Bool   Application_ownerPalette( Handle self, Bool set, Bool ownerPalette) { return false; }
-Bool   Application_clipOwner( Handle self, Bool set, Bool clipOwner)   { return true; }
+Bool   Application_clipChildren( Handle self, Bool set, Bool clip)   { return true; }
+Bool   Application_clipOwner( Handle self, Bool set, Bool clip_by_children)   { return false; }
 int    Application_tabOrder( Handle self, Bool set, int tabOrder)      { return 0; }
 SV   * Application_text    ( Handle self, Bool set, SV * text)       { return nilSV; }
 Bool   Application_transparent( Handle self, Bool set, Bool transparent) { return false; }
