@@ -266,7 +266,7 @@ Icon_mask( Handle self, Bool set, SV * svmask)
 		SV * sv = newSV_type(SVt_PV);
 		SvREADONLY_on(sv);
 		SvLEN_set(sv, 0); /* So Perl won't free it. */
-		SvPV_set(sv, var-> mask);
+		SvPV_set(sv, (char*)var-> mask);
 		SvCUR_set(sv, var-> maskSize);
 		SvPOK_only(sv);
 		return sv;
@@ -475,6 +475,8 @@ Icon_stretch( Handle self, int width, int height)
 	}      
 	inherited stretch( self, width, height);
 	free( var-> mask);
+
+
 	var->mask = newMask;
 	var->maskLine = lineSize;
 	var->maskSize = lineSize * abs( height);
@@ -670,6 +672,14 @@ Icon_extract( Handle self, int x, int y, int width, int height)
 	return h;
 }
 
+void
+Icon_make_empty( Handle self)
+{
+	inherited make_empty(self);
+	free( var->mask);
+	var->mask = nil;
+}
+
 Handle
 Icon_bitmap( Handle self)
 {
