@@ -163,11 +163,11 @@ void Window_handle_event( Handle self, PEvent event)
 			Event ev = *event;
 			ev. key. cmd   = cmTranslateAccel;
 			if ( !my-> message( self, &ev)) leave;
-			if ( my-> first_that( self, (void*)accel_notify, &ev)) leave;
+			if ( my-> first_that( self, (void*)prima_accel_notify, &ev)) leave;
 
 			ev. key. cmd    = cmDelegateKey;
 			ev. key. subcmd = 1;
-			if ( my-> first_that( self, (void*)accel_notify, &ev)) leave;
+			if ( my-> first_that( self, (void*)prima_accel_notify, &ev)) leave;
 		}
 		objCheck;
 		break;
@@ -668,7 +668,7 @@ Window_process_accel( Handle self, int key)
 {
 	return (
 		var-> modal ? 
-			my-> first_that_component( self, (void*)find_accel, &key)
+			my-> first_that_component( self, (void*)prima_find_accel, &key)
 			: inherited process_accel( self, key)
 		) != nilHandle;
 }
