@@ -143,6 +143,7 @@ static char* do_display = NULL;
 static int   do_debug   = 0;
 static Bool  do_icccm_only = false;
 static Bool  do_no_shmem   = false;
+static Bool  do_no_gtk   = false;
 
 static Bool
 init_x11( char * error_buf )
@@ -317,11 +318,15 @@ init_x11( char * error_buf )
 	guts. root = RootWindow( DISP, SCREEN);
 	guts. displaySize. x = DisplayWidth( DISP, SCREEN);
 	guts. displaySize. y = DisplayHeight( DISP, SCREEN);
+#ifdef HAVE_X11_XCURSOR_XCURSOR_H
+	guts. cursor_width = guts. cursor_height = XcursorGetDefaultSize(DISP);
+#else
 	XQueryBestCursor( DISP, guts. root,
 							guts. displaySize. x,     /* :-) */
 							guts. displaySize. y,
 							&guts. cursor_width,
 							&guts. cursor_height);
+#endif
 	XCHECKPOINT;
 	
 	TAILQ_INIT( &guts.paintq);
@@ -375,7 +380,7 @@ init_x11( char * error_buf )
 	if ( !prima_init_color_subsystem( error_buf)) return false;
 	if ( !prima_init_font_subsystem( error_buf)) return false;
 #ifdef WITH_GTK2
-	if (!prima_gtk_init()) return false;
+	guts. use_gtk = do_no_gtk ? false : ( prima_gtk_init() != NULL );
 #endif
 	bzero( &guts. cursor_gcv, sizeof( guts. cursor_gcv));
 	guts. cursor_gcv. cap_style = CapButt;
@@ -448,7 +453,10 @@ window_subsystem_get_options( int * argc, char *** argv)
 	"no-xft",        "do not use XFT",
 	"no-aa",         "do not anti-alias XFT fonts",
 	"font-priority", "match unknown fonts against: 'xft' (default) or 'core'",
-#endif   
+#endif
+#ifdef WITH_GTK2
+	"no-gtk",        "do not use GTK2",
+#endif
 	"font", 
 #ifdef USE_XFT
 				"default prima font in XLFD (-helv-misc-*-*-) or XFT(Helv-12) format",
@@ -488,6 +496,7 @@ window_subsystem_set_option( char * option, char * value)
 	} else if ( strcmp( option, "display") == 0) {
 		free( do_display);
 		do_display = duplicate_string( value);
+		setenv("DISPLAY", value, 1);
 		return true;
 	} else if ( strcmp( option, "icccm") == 0) {
 		if ( value) warn("`--icccm' option has no parameters");
@@ -497,6 +506,10 @@ window_subsystem_set_option( char * option, char * value)
 		if ( value) warn("`--no-shmem' option has no parameters");
 		do_no_shmem = true;
 		return true;
+	} else if ( strcmp( option, "no-gtk") == 0) {
+		if ( value) warn("`--no-gtk' option has no parameters");
+		do_no_gtk = true;
+		return true;
 	} else if ( strcmp( option, "debug") == 0) {
 		if ( !value) {
 			warn("`--debug' must be given parameters. `--debug=A` assumed\n");
@@ -546,7 +559,7 @@ window_subsystem_cleanup( void)
 	/*XXX*/
 	prima_end_menu();
 #ifdef WITH_GTK2
-	prima_gtk_done();
+	if ( guts. use_gtk) prima_gtk_done();
 #endif
 }
 
@@ -709,18 +722,23 @@ int
 apc_application_get_gui_info( char * description, int len)
 {
 #ifdef WITH_GTK2
-	if ( description) {
-		strncpy( description, "X Window System + GTK2", len);
-		description[len-1] = 0;
-	}
-	return guiGTK2;
+	if ( guts. use_gtk ) {
+		if ( description) {
+#ifdef WITH_GTK_NONX11
+			strncpy( description, "X Window System + GTK2", len);
 #else
+			strncpy( description, "X Window System + XQuartz + GTK2", len);
+#endif
+			description[len-1] = 0;
+		}
+		return guiGTK2;
+	}
+#endif
 	if ( description) {
 		strncpy( description, "X Window System", len);
 		description[len-1] = 0;
 	}
 	return guiXLib;
-#endif
 }
 
 Handle
