@@ -475,8 +475,8 @@ apc_dbm_get_handle( Handle self)
 	return (ApiHandle) X(self)-> gdrawable;
 }
 
-static Byte*
-mirror_bits( void)
+Byte*
+prima_mirror_bits( void)
 {
 	static Bool initialized = false;
 	static Byte bits[256];
@@ -513,7 +513,7 @@ prima_copy_xybitmap( unsigned char *data, const unsigned char *idata, int w, int
 			memcpy( ls*(h-y-1)+data, idata+y*ils, ls);
 		}
 	} else {
-		mirrored_bits = mirror_bits();
+		mirrored_bits = prima_mirror_bits();
 		for ( y = h-1; y >= 0; y--) {
 			register const unsigned char *s = idata+y*ils;
 			register unsigned char *t = ls*(h-y-1)+data;
@@ -527,7 +527,7 @@ prima_copy_xybitmap( unsigned char *data, const unsigned char *idata, int w, int
 void
 prima_mirror_bytes( unsigned char *data, int dataSize)
 {
-	Byte *mirrored_bits = mirror_bits();
+	Byte *mirrored_bits = prima_mirror_bits();
 	while ( dataSize--) {
 		*data = mirrored_bits[*data];
 		data++;
@@ -2805,6 +2805,16 @@ apc_gp_stretch_image( Handle self, Handle image,
 			return false;
 		}
 		XDestroyImage( i);
+		if ( src == SRC_BITMAP && !XT_IS_IMAGE(YY)) {
+			PImage o = (PImage) obj;
+			o->type = imbpp1;
+			o->palette[0].r = XX->fore. color & 0xff;
+			o->palette[0].g = (XX->fore. color >> 8) & 0xff;
+			o->palette[0].b = (XX->fore. color >> 16) & 0xff;
+			o->palette[1].r = XX->back. color & 0xff;
+			o->palette[1].g = (XX->back. color >> 8) & 0xff;
+			o->palette[1].b = (XX->back. color >> 16) & 0xff;
+		}
 		ok = apc_gp_stretch_image( self, obj, dst_x, dst_y, 0, 0, dst_w, dst_h, src_w, src_h, rop);
 	} else if ( src == SRC_LAYERED ) {
 		obj = ( Handle) create_object("Prima::Icon", "");
@@ -2814,13 +2824,15 @@ apc_gp_stretch_image( Handle self, Handle image,
 			return false;
 		}
 		ok = apc_gp_stretch_image( self, obj, dst_x, dst_y, 0, 0, dst_w, dst_h, src_w, src_h, rop);
-	} else {
+	} else if ( img->w != dst_w || img->h != dst_h || src_x != 0 || src_y != 0) {
 		/* extract local bits */
 		obj = CImage(image)->extract( image, src_x, src_y, src_w, src_h );
 		if ( !obj ) return false;
 		CImage(obj)-> set_scaling( obj, istBox );
 		CImage(obj)-> stretch( obj, dst_w, dst_h );
 		ok  = apc_gp_put_image( self, obj, dst_x, dst_y, 0, 0, dst_w, dst_h, rop);
+	} else {
+		return apc_gp_put_image( self, obj, dst_x, dst_y, 0, 0, dst_w, dst_h, rop);
 	}
 
 	Object_destroy( obj );
